/*
* Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*/

#include "main.h"
#include <string.h>

#define ERROR(format, arg...) LOGE(format, ##arg)
#define TEXT_PARTS_NUMBER 50

Evas_Object *GLOBAL_ENTRY_BOX;
Evas_Object *LOG_SIGNAL_BOX;
Evas_Object *LOG_TEXT_BOX;

static void add_log_info(char *object_text, char *signal)
{
	elm_entry_cursor_begin_set(LOG_SIGNAL_BOX);
	elm_entry_entry_insert(LOG_SIGNAL_BOX, signal);
	elm_entry_entry_insert(LOG_SIGNAL_BOX, "<br>");

	elm_entry_cursor_begin_set(LOG_TEXT_BOX);
	elm_entry_entry_insert(LOG_TEXT_BOX, object_text);
	elm_entry_entry_insert(LOG_TEXT_BOX, "<br>");
}

static void say_smart_cb(void *data, Evas_Object *obj, void *event_info)
{
	char *signal = data;
	char *object_text = elm_object_text_get(obj);

	add_log_info(object_text, signal);
}

void say_cb(void *data, const char *signal)
{
	add_log_info("ENTRY", signal);
}

void non_discardable_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_atspi_bridge_utils_say(elm_entry_entry_get(GLOBAL_ENTRY_BOX), EINA_FALSE, say_cb, NULL);
}

void discardable_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_atspi_bridge_utils_say(elm_entry_entry_get(GLOBAL_ENTRY_BOX), EINA_TRUE, say_cb, NULL);
}

void clean_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_entry_entry_set(GLOBAL_ENTRY_BOX, "");
}

Evas_Object *create_entry(Evas_Object *layout, char *swallow_part)
{
	Evas_Object *display_window = elm_entry_add(layout);

	if (display_window != NULL) {
		evas_object_size_hint_align_set(display_window, EVAS_HINT_FILL, EVAS_HINT_FILL);
		evas_object_size_hint_weight_set(display_window, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		elm_object_part_content_set(layout, swallow_part, display_window);
		evas_object_show(display_window);

		elm_entry_editable_set(display_window, EINA_FALSE);
		elm_entry_scrollable_set(display_window, EINA_TRUE);

		elm_scroller_policy_set(display_window, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_ON);
	}

	return display_window;
}

Evas_Object *create_button(Evas_Object *layout,char *object_text, char *part_name, Evas_Smart_Cb cb)
{
	Evas_Object *btn = elm_button_add(layout);

	if (btn != NULL) {
		elm_object_text_set(btn, object_text);
		elm_object_style_set(btn, "default");
		elm_object_part_content_set(layout, part_name, btn);

		evas_object_smart_callback_add(btn, "clicked", cb, NULL);
		evas_object_smart_callback_add(btn, "access,read,cancel", (Evas_Smart_Cb) say_smart_cb, "access,read,cancel");
		evas_object_smart_callback_add(btn, "access,read,stop", (Evas_Smart_Cb) say_smart_cb, "access,read,stop");
		evas_object_smart_callback_add(btn, "access,read,skip", (Evas_Smart_Cb) say_smart_cb, "access,read,skip");
	}

	return btn;
}

void add_sample_text()
{
	char *short_text = "This program reads the text and displays it in entry. Click some button to see what happens. ";
	int len = strlen(short_text);
	char *long_text = malloc(sizeof(char) * (len+4) * TEXT_PARTS_NUMBER + 1);
	int i;
	int size = 0;

	for (i = 0; i < TEXT_PARTS_NUMBER; i++) {
		memcpy(long_text+size, short_text, len);
		size += len;
		sprintf(long_text+size, " %2d ", i+1);
		size += 4;
	}

	elm_entry_entry_set(GLOBAL_ENTRY_BOX, long_text);

	free(long_text);
}

static Evas_Object
*create_layout(Evas_Object *parent)
{
	Evas_Object *layout;
	Evas_Object *btn;

	layout = elm_layout_add(parent);
	if (layout == NULL) {
		ERROR("elm_layout_add() failed");
		return NULL;
	}

	elm_layout_file_set(layout, ELM_DEMO_EDJ, "direct_reading_layout");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	btn = create_button(layout, "Non-Discardable", "top_left_button", non_discardable_cb);
	if (btn == NULL) {
		ERROR("elm_button_add() for Non-Discardable button failed");
		return NULL;
	}

	btn = create_button(layout, "Discardable", "top_right_button", discardable_cb);
	if (btn == NULL) {
		ERROR("elm_button_add() for Discardable button failed");
		return NULL;
	}

	btn = create_button(layout, "CLEAN", "bottom_button", clean_cb);
	if (btn == NULL) {
		ERROR("elm_button_add() for CLEAN button failed");
		return NULL;
	}

	GLOBAL_ENTRY_BOX = create_entry(layout, "screen");
	if (GLOBAL_ENTRY_BOX == NULL) {
		ERROR("elm_entry_add() for GLOBAL_ENTRY_BOX failed");
		return NULL;
	}

	LOG_SIGNAL_BOX = create_entry(layout, "log_info_signal");
	if (LOG_SIGNAL_BOX == NULL) {
		ERROR("elm_entry_add() for LOG_SIGNAL_BOX failed");
		return NULL;
	}

	LOG_TEXT_BOX = create_entry(layout, "log_info_text");
	if (LOG_TEXT_BOX == NULL) {
		ERROR("elm_entry_add() for LOG_TEXT_BOX failed");
		return NULL;
	}

	elm_entry_editable_set(GLOBAL_ENTRY_BOX, EINA_TRUE);

	add_sample_text();

	return layout;
}

void
direct_reading_cb(void *data, Evas_Object *obj, void *event_info)
{
	Evas_Object *layout;
	Evas_Object *nf = data;

	layout = create_layout(nf);

	if (layout != NULL)
		elm_naviframe_item_push(nf, "Direct Reading", NULL, NULL, layout, NULL);
	else
		ERROR("UI creation failed");
}
